# Contributing to WESTPA  
The following is also available on our wiki's [Developer's Guide](https://github.com/westpa/westpa/wiki/WESTPA-Developer%27s-Guide).

## Introduction

Thank you for your interest in contributing to the WESTPA software package! This guide is intended to facilitate your coding efforts by outlining the organization of the WESTPA code and the development process that should be followed. Contributions may involve either directly adding features to the WESTPA code or [Writing Plugins](https://github.com/westpa/westpa/wiki/Writing-Plugins). 

As a start, the following video discussions involving WESTPA development may be helpful to you:
- [Overview of WESTPA 1.0](https://www.youtube.com/watch?v=YpltPzpcmLY&amp;ab_channel=WESTPATutorials)
- [WESTPA 1.0 code Q&A](https://youtu.be/EIpBzpGvcY4)
- [Plans for a WESTPA-Orion Interface](https://youtu.be/vcNPGqyJw0k)
- [Overview of WESTPA 2.0](https://www.youtube.com/watch?v=yS7dfUi7oWU&list=PLLfi4uMcAMv4mKDvFf9EbELOucT40Pmuv&index=6&t=0s)

Other resources include our [GitHub Issue Tracker](https://github.com/westpa/westpa/issues), archives of the [WESTPA developer's mailing list](https://groups.google.com/group/westpa-devel), and our [Sphinx page](https://westpa.github.io/westpa/sphinx_index.html) for API documentation of the WESTPA code from autogenerated docstrings. Please note our [Code of Conduct](https://github.com/westpa/westpa/wiki/Code-of-Conduct). 

## Organization of WESTPA

**Figure 1 | Key steps of running a weighted ensemble simulation within the WESTPA framework.** Upon initialization of the WESTPA environment and trajectories, the simulation manager controls the execution of a weighted ensemble simulation, including dynamics propagation using a selected dynamics engine via the work manager.

![Organization of Westpa](https://raw.githubusercontent.com/westpa/westpa/gh-pages/images/westpa_map_simple.png)

- [HDF5 File Organization of Simulation Data](https://github.com/westpa/westpa/wiki/HDF5-File-Organization-of-Simulation-Data)

- [Index of Command-Line Tools](https://github.com/westpa/westpa/wiki/User-Guide#INDEX_OF_COMMANDLINE_TOOLS)


## Guidelines
- All contributions are submitted by making Pull Requests, *even from developers who have write permission on the WESTPA repository.* 
- All code should pass the [GitHub Actions continuous integration (CI) tests](https://github.com/westpa/westpa/actions).
- All code should be documented (Section 5).
- No changes are ever committed without review and approval by a core WESTPA developer. Every effort will be made to respond to a Pull Request (PR) within a week. 

## Development process

### 1. Post your coding plans
To post your coding plans, please open an issue or comment on a relevant issue on [GitHub](https://github.com/westpa/westpa/issues). To avoid any duplication of efforts, @ltchong may contact you to set up a videoconferencing call with other developers to discuss your plans. 

### 2. Develop your contribution
If @ltchong or @mczwier confirms that your coding plans are not duplicating existing efforts, create your own copy of the WESTPA repository by clicking on the Fork button on [GitHub](https://github.com/westpa/westpa) and develop your contribution in this separate fork. To adhere to the 
[Stylistic Guidelines](https://github.com/westpa/westpa/wiki/Stylistic-Guidelines), make sure you have [pre-commit](https://pre-commit.com/) configured and test your contribution locally using [pytest](https://docs.pytest.org/en/stable).

All [GitHub Actions continuous integration (CI) tests](https://github.com/westpa/westpa/actions), including unit/functional tests and checks on coding style must pass before submitting your contribution. If any of the CI tests fail, you can find out why by clicking on the “failed” icon (red cross) and inspecting the build and test log. To avoid overuse and waste of this resource, please test your work locally before committing.

All documentation must build properly with Read the Docs. Please check code documentation before committing.

### 3. Submit your contribution
Submit your contribution by making a PR. If your contribution introduces a new feature or changes functionality, please explain in a PR comment. 

### 4. Review process
Reviewers (other developers and interested community members) will comment inline on your PR to help you improve its implementation, documentation, and style. Every single developer working on the project has their code reviewed with the intention of a friendly conversation from which we all learn to improve the quality of the project. 

To update your PR, make your changes on your fork, commit, run tests, and only if they succeed, push to your fork. As soon as those changes are pushed up (to the same branch as before) the PR will update automatically. The [GitHub Actions CI services](https://github.com/westpa/westpa/actions) are triggered after each PR update. If you do not know how to fix the test failures, you may still push your changes and ask for help in a PR comment. 

A PR must be approved by at least one core WESTPA developer before merging. Approval means the core developer has carefully reviewed the changes, and the PR is ready for merging.

### 5. Document changes
Beyond changes to a functions docstring and possible description in the general documentation, if your change introduces any user-facing modifications they may need to be mentioned in the release notes. To add your change to the release notes, you need to create a short file with a summary and place it in CHANGELOG. The file README.rst details the format and filename conventions. If your change introduces a deprecation, make sure to discuss this first on [GitHub](https://github.com/westpa/westpa/issues).

### 6. Cross referencing issues
If the PR relates to any issues, you can add the text xref gh-xxxx where xxxx is the number of the issue to GitHub comments. Likewise, if the PR solves an issue, replace the xref with closes, fixes or any of the other flavors GitHub accepts. In the source code, be sure to preface any issue or PR reference with gh-xxxx.


## Useful Guides

- [Stylistic Guidelines](https://github.com/westpa/westpa/wiki/Stylistic-Guidelines)
- [Test Coverage](https://github.com/westpa/westpa/wiki/Test-Coverage)
- [Building docs](https://github.com/westpa/westpa/wiki/Building-docs)
- [Writing Plugins](https://github.com/westpa/westpa/wiki/Writing-Plugins)
